import pygame
from control.block import *
import time
import math
import constants
class game_object:
	# superclass of all interactive game objects
	def __init__(self, screen, name, x, y, width, height, player_controlled, is_movable, is_real):
		self.screen = screen
		self.name = name # for iteration logic
		self.player_controlled = player_controlled	# is this object controlled by the player?
		self.is_movable = is_movable	# does this object respond to physics?
		self.is_real = is_real	# does this object supply physics?
		self.is_super = False #is this a super saiyan!!?
		# not all fields are implemented(but MOST are!)
		self.frames_standing_right = []
		self.frames_standing_left = []
		self.frames_running_right = []
		self.frames_running_left = []
		self.frames_jumping_right = []
		self.frames_jumping_left = []
		self.frames_super_standing_right = []
		self.frames_super_standing_left = []
		self.frames_super_running_right = []
		self.frames_super_running_left = []
		self.frames_super_jumping_right = []
		self.frames_super_jumping_left = []
		self.frames_falling_right = []
		self.frames_falling_left = []
		self.current_frame_index = 0
		self.since_last_frame = 0
		self.frame_change = True
		self.frames_fps = 4
		self.current_frame = None
		self.gravity = 1 
		self.mass = 1
		self.torque = 0
		self.traction = 3
		self.prev_direction_x = 0
		self.prev_direction_y = 0
		self.direction_x = 0
		self.direction_y = 0
		self.acceleration_x = 0
		self.acceleration_y = 0
		self.acceleration_increment_x = 0
		self.acceleration_increment_y = 0
		self.default_acceleration_increment_x = 0
		self.default_acceleration_increment_y = 0
		self.jump_acceleration = 0
		self.max_acceleration_x = 20
		self.max_acceleration_y = 0
		self.default_max_acceleration_x = 0
		self.default_max_acceleration_y = 0
		self.grounded = False
		self.jumping = False
		self.has_jump = False
		self.wall_left = False
		self.wall_right = False
		self.velocity_x = 0
		self.velocity_y = 0
		self.terminal_velocity_y = 10
		self.terminal_velocity_x = 10
		self.hp = 100
		self.mana = 0
		self.moves_taken = 0
		self.effect = "none"
		self.block = block()
		self.block.width = width
		self.block.height = height
		self.x = x
		self.y = y
		self.block.set_screen(screen)
	
	def control(self):
		self.player_controlled = True
	
		#set the vector of this game object(only activated if movable)
	def move(self,x,y):
		if self.prev_direction_x != x:
			self.frame_change = True
			self.prev_direction_x = x
			self.wall_left = False
			self.wall_right = False
		if (self.prev_direction_x == x and (self.wall_left or self.wall_right)) == False:
			if x != 0:
				if x > 0:
					self.direction_x = 1
				elif x < 0:
					self.direction_x = -1
				self.acceleration_increment_x = self.default_acceleration_increment_x
			else:
				self.acceleration_x = 1
				self.acceleration_increment_x = 0
				if self.velocity_x > 0:
					self.velocity_x -= 2
				
		if y != 0 and self.grounded == True:
			if y > 0:
				self.direction_y = 1
			elif y < 0 and self.mana > 0:
				self.has_jump = True
				self.direction_y = -1
				if self.jumping == False:
					self.jumping = True
					self.mana -= 1
					self.frame_change = True
					#self.is_super = False
			self.acceleration_increment_y = self.default_acceleration_increment_y
		
		# increase the speed at which an object builds horizontal velocity
	def accelerate_x(self):
		self.acceleration_x += self.acceleration_increment_x
		if self.acceleration_x > self.max_acceleration_x:
			self.acceleration_x = self.max_acceleration_x
			
		# increase the speed at which an object builds vertical velocity
	def accelerate_y(self):
		self.acceleration_y += self.acceleration_increment_y
		if self.acceleration_y > self.max_acceleration_y:
			self.acceleration_y = self.max_acceleration_y
			if self.direction_y == -1:
				self.direction_y = 1
				self.grounded = False
				self.reset_jump()
				self.frame_change = True
				self.acceleration_y = 1
			
		# calculate new velocity	
	def set_velocity(self):
		self.velocity_x += self.acceleration_x
		self.velocity_y += self.acceleration_y
		if self.mana > 0:
			self.moves_taken += int(round(self.velocity_x / 10))
			self.moves_taken += int(round(self.velocity_y/10))
		
		if self.terminal_velocity_x < self.velocity_x:
			self.velocity_x = self.terminal_velocity_x
			
		if self.terminal_velocity_y < self.velocity_y:
			self.velocity_y = self.terminal_velocity_y
		
		if self.direction_y == -1 or self.grounded == False:
			self.velocity_x -= self.mass
		if self.velocity_x < 0:
			self.velocity_x = 0
			
		# calculate where the object will be drawn next
	def calculate_pos(self, viewport_offset_x, viewport_offset_y):
		new_x = self.x + (self.velocity_x * self.direction_x)
		new_y = self.y + (self.velocity_y * self.direction_y)
		if self.wall_left == False and self.wall_right == False:
			self.set_pos(new_x,new_y, viewport_offset_x, viewport_offset_y)
		else:
			self.set_pos(self.x, new_y, viewport_offset_x, viewport_offset_y)
	
	def reset_jump(self):
		self.jumping = False
		self.max_acceleration_y = self.default_max_acceleration_y
	
	def set_pos(self,x,y, viewport_x, viewport_y):
		self.x = x
		self.y = y
		self.block.set_pos(x + viewport_x,y + viewport_y)
		
	def draw(self):
		self.draw_me()
		self.block.image = self.current_frame
		self.block.draw()
		
	def draw_me(self):
		current_animation = None
		if self.frame_change == True:			
			self.current_frame_index = 0
			self.since_last_frame = 0
			self.frame_change = False
		else:
			self.since_last_frame += 1
		
		if self.direction_x == 1:
			if self.jumping == True:
				if self.mana > 0:
					current_animation = self.frames_super_jumping_right
				else:
					current_animation = self.frames_jumping_right
			elif self.velocity_x > 0:
				if self.mana > 0:
					current_animation = self.frames_super_running_right
				else:
					current_animation = self.frames_running_right
			else:
				if self.mana > 0:
					current_animation = self.frames_super_standing_right
				else:
					current_animation = self.frames_standing_right
		elif self.direction_x == -1:
			if self.jumping == True:
				if self.mana > 0:
					current_animation = self.frames_super_jumping_left
				else:
					current_animation = self.frames_jumping_left
			elif self.velocity_x > 0:
				if self.mana > 0:
					current_animation = self.frames_super_running_left
				else:
					current_animation = self.frames_running_left
			else:
				if self.mana > 0:
					current_animation = self.frames_super_standing_left
				else:
					current_animation = self.frames_standing_left
		if current_animation != None and len(current_animation) > 0:
			if self.since_last_frame >= constants.TARGET_FPS / self.frames_fps:
				self.since_last_frame = 0
				self.current_frame_index += 1
				
				if len(current_animation) <= self.current_frame_index:
					self.current_frame_index = 0
				self.current_frame = current_animation[self.current_frame_index]
		else:
			self.current_frame = None